package com.shigongli.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.shigongli.common.WxUtils;
import com.shigongli.config.WeixinConfig;
import com.shigongli.entity.TaHouse;
import com.shigongli.entity.TaHousePerson;
import com.shigongli.entity.TaShopKeeper;
import com.shigongli.mapper.TaHousePersonMapper;
import com.shigongli.service.ITaHousePersonService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.shigongli.service.ITaShopKeeperService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.subscribe.WxMpSubscribeMessage;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 住房人 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-12-11
 */
@Slf4j
@Service
public class TaHousePersonServiceImpl extends ServiceImpl<TaHousePersonMapper, TaHousePerson> implements ITaHousePersonService {
    @Autowired
    ITaShopKeeperService iTaShopKeeperService;

    @Autowired
    WxUtils wxUtils;

    @Autowired
    WeixinConfig weixinConfig;

    @Async
    @Override
    public void sendMessageToManager(TaHouse taHouse, String dateStr) {
        try {
            String houseId = taHouse.getHouseId();
            List<TaShopKeeper> taShopKeeperList = iTaShopKeeperService.getByHouse(houseId);
            if (null == taShopKeeperList || taShopKeeperList.size() < 1) {
                log.error("没有找到房源【{}】对应的店主", houseId);
                return;
            }

            // 先暂时发送一个店主
            TaShopKeeper taShopKeeper = taShopKeeperList.get(0);

            // 找到消息模板
            List<WeixinConfig.Ma.SubscribeMessage> subscribeMessages = weixinConfig.getMa().getSubscribeMessages();
            WeixinConfig.Ma.SubscribeMessage subscribeMessage = null;
            if (null == subscribeMessages || subscribeMessages.size() < 1) {
                log.error("没有找到小程序消息提醒配置");
                return;
            }
            for (WeixinConfig.Ma.SubscribeMessage message : subscribeMessages) {
                if ("settled-in".equals(message.getType())) {
                    subscribeMessage = message;
                    break;
                }
            }
            if (null == subscribeMessage) {
                log.error("没有找到入店提醒的消息配置");
                return;
            }

            // 发送内容
            List<WxMaSubscribeMessage.Data> dataList = new ArrayList<>();
            dataList.add(new WxMaSubscribeMessage.Data("thing6", taHouse.getTitle().substring(0, 10)));
            dataList.add(new WxMaSubscribeMessage.Data("date2", dateStr));

            //
            WxMaService maService = wxUtils.getMaService();
            WxMaSubscribeMessage wxMaSubscribeMessage = WxMaSubscribeMessage
                    .builder()
                    .templateId(subscribeMessage.getCode())
                    .toUser(taShopKeeper.getOpenid())
                    .page(String.format("/pages/house/household/index?houseId=%s", houseId))
                    .data(dataList)
                    .build();

            maService.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);

        } catch (Exception e) {
            log.error("发送订阅消息给店主失败: {}", e.getMessage());
            e.printStackTrace();
        }
    }
}
